# SPDX-FileCopyrightText: Contributors to the Power Grid Model project <powergridmodel@lfenergy.org>
#
# SPDX-License-Identifier: MPL-2.0

"""
Power grid model (de)serialization
"""

from abc import ABC, abstractmethod
from ctypes import byref
from enum import IntEnum

from power_grid_model._core.dataset_definitions import DatasetType, _map_to_component_types, _str_to_datatype
from power_grid_model._core.error_handling import assert_no_error
from power_grid_model._core.index_integer import IdxC
from power_grid_model._core.power_grid_core import (
    CharPtr,
    DeserializerPtr,
    SerializerPtr,
    WritableDatasetPtr,
    power_grid_core as pgc,
)
from power_grid_model._core.power_grid_dataset import CConstDataset, CWritableDataset
from power_grid_model.data_types import Dataset
from power_grid_model.errors import PowerGridSerializationError
from power_grid_model.typing import ComponentAttributeMapping


class SerializationType(IntEnum):
    """Serialization Format Types"""

    JSON = 0
    MSGPACK = 1


class Deserializer:
    """
    Deserializer for the Power grid model
    """

    _deserializer: DeserializerPtr
    _dataset_ptr: WritableDatasetPtr
    _dataset: CWritableDataset
    _data_filter: ComponentAttributeMapping

    def __new__(
        cls,
        data: str | bytes,
        serialization_type: SerializationType,
        data_filter: ComponentAttributeMapping,
    ):
        instance = super().__new__(cls)

        raw_data = data if isinstance(data, bytes) else data.encode()
        instance._deserializer = pgc.create_deserializer_from_binary_buffer(
            raw_data, len(raw_data), serialization_type.value
        )
        assert_no_error()

        instance._dataset_ptr = pgc.deserializer_get_dataset(instance._deserializer)
        assert_no_error()

        instance._data_filter = data_filter
        instance._dataset = CWritableDataset(instance._dataset_ptr, data_filter=data_filter)
        assert_no_error()

        return instance

    def __del__(self):
        if hasattr(self, "_deserializer"):
            pgc.destroy_deserializer(self._deserializer)

    def load(self) -> Dataset:
        """
        Load the deserialized data to a new dataset.

        Raises:
            ValueError: if the data is inconsistent with the rest of the dataset or a component is unknown.
            PowerGridError: if there was an internal error.

        Returns:
            A tuple containing the deserialized dataset in Power grid model input format and the type of the dataset.
        """
        pgc.deserializer_parse_to_buffer(self._deserializer)
        return self._dataset.get_data()


class Serializer(ABC):
    """
    Serializer for the Power grid model
    """

    _data: Dataset
    _dataset: CConstDataset
    _serializer: SerializerPtr

    def __new__(cls, data: Dataset, serialization_type: SerializationType, dataset_type: DatasetType | None = None):
        instance = super().__new__(cls)

        instance._data = data
        instance._dataset = CConstDataset(instance._data, dataset_type=dataset_type)
        assert_no_error()

        instance._serializer = pgc.create_serializer(instance._dataset.get_dataset_ptr(), serialization_type.value)
        assert_no_error()

        return instance

    def __del__(self):
        if hasattr(self, "_serializer"):
            pgc.destroy_serializer(self._serializer)

    def dump_str(self, *, use_compact_list: bool = False, indent: int = 2) -> str:
        """
        Dump the data to a decoded str; if supported.

        Args:
            use_compact_list: whether or not to use compact lists (sparse data). Defaults to False.
            indent:
                use specified indentation to make data more readable
                Use 0 or negative value for no indentation. Defaults to 2

        Returns:
            A serialized string containing the dataset.
        """
        data = pgc.serializer_get_to_zero_terminated_string(self._serializer, int(use_compact_list), indent)
        assert_no_error()
        return data

    def dump_bytes(self, *, use_compact_list: bool = False) -> bytes:
        """
        Dump the data to a bytes object; if supported.

        Args:
            use_compact_list (bool, optional): whether or not to use compact lists (sparse data). Defaults to False.

        Returns:
            The raw bytes of the serialization of the datast.
        """
        raw_data = CharPtr()  # pylint: disable(not-callable)
        size = IdxC()  # pylint: disable(not-callable)
        pgc.serializer_get_to_binary_buffer(self._serializer, int(use_compact_list), byref(raw_data), byref(size))
        assert_no_error()

        result = raw_data[: size.value]
        if not isinstance(result, bytes):
            raise PowerGridSerializationError("Invalid output data type")

        return result

    @abstractmethod
    def dump(self, *args, **kwargs):
        """
        Dump the data in the recommended data format (e.g. str or bytes, depending on the serialization_type).

        Args:
            *: any serialization_type dependent args
        """


class _BytesSerializer(Serializer):
    """
    Base type for serialization to bytes
    """

    dump = Serializer.dump_bytes


class _StringSerializer(Serializer):
    """
    Base type for serialization to str
    """

    dump = Serializer.dump_str


class JsonDeserializer(Deserializer):  # pylint: disable=too-few-public-methods
    """
    JSON deserializer for the Power grid model
    """

    def __new__(cls, data: str | bytes, data_filter: ComponentAttributeMapping):
        return super().__new__(cls, data, SerializationType.JSON, data_filter=data_filter)


class MsgpackDeserializer(Deserializer):  # pylint: disable=too-few-public-methods
    """
    msgpack deserializer for the Power grid model
    """

    def __new__(cls, data: bytes, data_filter: ComponentAttributeMapping):
        return super().__new__(cls, data, SerializationType.MSGPACK, data_filter=data_filter)


class JsonSerializer(_StringSerializer):  # pylint: disable=too-few-public-methods
    """
    JSON deserializer for the Power grid model
    """

    def __new__(cls, data: Dataset, dataset_type: DatasetType | None = None):
        return super().__new__(cls, data, SerializationType.JSON, dataset_type=dataset_type)


class MsgpackSerializer(_BytesSerializer):  # pylint: disable=too-few-public-methods
    """
    msgpack deserializer for the Power grid model
    """

    def __new__(cls, data: Dataset, dataset_type: DatasetType | None = None):
        return super().__new__(cls, data, SerializationType.MSGPACK, dataset_type=dataset_type)


def json_deserialize(
    data: str | bytes,
    data_filter: ComponentAttributeMapping = None,
) -> Dataset:
    """
    Load serialized JSON data to a new dataset.

    Args:
        data: the data to deserialize.
        data_filter: the data filter to apply to the dataset.

    Raises:
        ValueError: if the data is inconsistent with the rest of the dataset or a component is unknown.
        PowerGridError: if there was an internal error.

    Returns:
        A tuple containing the deserialized dataset in Power grid model input format and the type of the dataset.
    """
    result = JsonDeserializer(data, data_filter=data_filter).load()
    assert_no_error()
    return result


def json_serialize(
    data: Dataset,
    dataset_type: DatasetType | None = None,
    use_compact_list: bool = False,
    indent: int = 2,
) -> str:
    """
    Dump data to a JSON str.

    If the dataset_type is not specified or None, it will be deduced from the dataset if possible.
    Deduction is not possible in case data is empty.

    Args:
        data: the dataset
        dataset_type: the type of the dataset. Defaults to None. Required str-type if data is empty.
        use_compact_list: whether or not to use compact lists (sparse data). Defaults to False.
        indent:
            use specified indentation to make data more readable
            Use 0 or negative value for no indentation. Defaults to 2

    Raises:
        PowerGridError: if there was an internal error.

    Returns:
        A serialized string containing the dataset.
    """
    data = _map_to_component_types(data)
    if dataset_type is not None:
        dataset_type = _str_to_datatype(dataset_type)
    result = JsonSerializer(data=data, dataset_type=dataset_type).dump(use_compact_list=use_compact_list, indent=indent)
    assert_no_error()
    return result


def msgpack_deserialize(data: bytes, data_filter: ComponentAttributeMapping = None) -> Dataset:
    """
    Load serialized msgpack data to a new dataset.

    Args:
        data: the data to deserialize.

    Raises:
        ValueError: if the data is inconsistent with the rest of the dataset or a component is unknown.
        PowerGridError: if there was an internal error.

    Returns:
        A tuple containing the deserialized dataset in Power grid model input format and the type of the dataset.
    """
    result = MsgpackDeserializer(data, data_filter=data_filter).load()
    assert_no_error()
    return result


def msgpack_serialize(
    data: Dataset,
    dataset_type: DatasetType | None = None,
    use_compact_list: bool = False,
) -> bytes:
    """
    Dump the data to raw msgpack bytes.

    If the dataset_type is not specified or None, it will be deduced from the dataset if possible.
    Deduction is not possible in case data is empty.

    Args:
        data: the dataset
        dataset_type: the type of the dataset. Defaults to None. Required str-type if data is empty.
        use_compact_list: whether or not to use compact lists (sparse data). Defaults to False.

    Raises:
        KeyError: if the dataset_type was not provided and could not be deduced from the dataset (i.e. it was empty).
        PowerGridError: if there was an internal error.

    Returns:
        A serialized string containing the dataset.
    """
    data = _map_to_component_types(data)
    if dataset_type is not None:
        dataset_type = _str_to_datatype(dataset_type)
    result = MsgpackSerializer(data=data, dataset_type=dataset_type).dump(use_compact_list=use_compact_list)
    assert_no_error()
    return result



# SPDX-FileCopyrightText: Contributors to the Power Grid Model project <powergridmodel@lfenergy.org>
#
# SPDX-License-Identifier: MPL-2.0

"""
This module contains functions that may be useful when working with the power-grid-model library.
"""

import json
import math
import tempfile
import warnings
from pathlib import Path
from typing import cast as cast_type

import numpy as np

from power_grid_model import CalculationMethod, PowerGridModel
from power_grid_model._core.dataset_definitions import DatasetType, _map_to_component_types
from power_grid_model._core.serialization import (  # pylint: disable=unused-import
    json_deserialize,
    json_serialize,
    msgpack_deserialize,
    msgpack_serialize,
)
from power_grid_model._utils import (
    _extract_data_from_component_data,
    _extract_indptr,
    get_and_verify_batch_sizes as _get_and_verify_batch_sizes,
    get_batch_size as _get_batch_size,
    get_dataset_type,
    is_columnar,
    is_sparse,
)
from power_grid_model.data_types import (
    BatchComponentData,
    BatchDataset,
    Dataset,
    DenseBatchArray,
    IndexPointer,
    SingleComponentData,
    SingleDataset,
)
from power_grid_model.errors import PowerGridError, PowerGridSerializationError
from power_grid_model.typing import ComponentAttributeMapping

_DEPRECATED_FUNCTION_MSG = "This function is deprecated."
_DEPRECATED_JSON_DESERIALIZATION_MSG = f"{_DEPRECATED_FUNCTION_MSG} Please use json_deserialize_to_file instead."
_DEPRECATED_JSON_SERIALIZATION_MSG = f"{_DEPRECATED_FUNCTION_MSG} Please use json_serialize_from_file instead."


def get_dataset_scenario(dataset: BatchDataset, scenario: int) -> SingleDataset:
    """
    Obtain the single dataset at a given scenario, independently of the internal batch data structure.

    Args:
        dataset: the batch dataset
        scenario: the scenario index

    Raises:
        IndexError: if the scenario is out of range for any of the components.

    Returns:
        The dataset for a specific scenario
    """

    def _get_dense_scenario(arr: np.ndarray) -> np.ndarray:
        return arr[scenario]

    def _get_sparse_scenario(arr: np.ndarray, indptr: IndexPointer) -> np.ndarray:
        return arr[indptr[scenario] : indptr[scenario + 1]]

    def _get_component_scenario(component_scenarios: BatchComponentData) -> SingleComponentData:
        data = _extract_data_from_component_data(component_scenarios)

        if is_sparse(component_scenarios):
            indptr = _extract_indptr(component_scenarios)
            if is_columnar(component_scenarios):
                return {
                    attribute: _get_sparse_scenario(attribute_data, indptr)
                    for attribute, attribute_data in data.items()
                }
            return _get_sparse_scenario(data, indptr)

        if is_columnar(component_scenarios):
            return {attribute: _get_dense_scenario(attribute_data) for attribute, attribute_data in data.items()}
        return _get_dense_scenario(cast_type(DenseBatchArray, component_scenarios))

    return {component: _get_component_scenario(component_data) for component, component_data in dataset.items()}


def get_dataset_batch_size(dataset: BatchDataset) -> int:
    """
    Get the number of scenarios in the batch dataset.

    Args:
        dataset: the batch dataset

    Raises:
        ValueError: if the batch dataset is inconsistent.

    Returns:
        The size of the batch dataset. Making use of existing _utils function.
    """
    return _get_and_verify_batch_sizes(dataset)


def get_component_batch_size(data_array: BatchComponentData) -> int:
    """
    Determine the number of batch scenarios and verify the data structure

    Args:
        data_array: batch data for power-grid-model

    Returns:
        The number of batch scenarios in data_array
    """
    return _get_batch_size(data_array)


def json_deserialize_from_file(
    file_path: Path,
    data_filter: ComponentAttributeMapping = None,
) -> Dataset:
    """
    Load and deserialize a JSON file to a new dataset.

    Args:
        file_path: the path to the file to load and deserialize.

    Raises:
        ValueError: if the data is inconsistent with the rest of the dataset or a component is unknown.
        PowerGridError: if there was an internal error.

    Returns:
        The deserialized dataset in Power grid model input format.
    """
    with open(file_path, encoding="utf-8") as file_pointer:
        return json_deserialize(file_pointer.read(), data_filter=data_filter)


def json_serialize_to_file(
    file_path: Path,
    data: Dataset,
    dataset_type: DatasetType | None = None,
    use_compact_list: bool = False,
    indent: int | None = 2,
):
    """
    Export JSON data in most recent format.

    Args:
        file_path: the path to the file to load and deserialize.
        data: a single or batch dataset for power-grid-model.
        use_compact_list: write components on a single line.
        indent: indent of the file. Defaults to 2.

    Returns:
        Save to file.
    """
    data = _map_to_component_types(data)
    result = json_serialize(
        data=data, dataset_type=dataset_type, use_compact_list=use_compact_list, indent=-1 if indent is None else indent
    )

    with open(file_path, mode="w", encoding="utf-8") as file_pointer:
        file_pointer.write(result)


def msgpack_deserialize_from_file(
    file_path: Path,
    data_filter: ComponentAttributeMapping = None,
) -> Dataset:
    """
    Load and deserialize a msgpack file to a new dataset.

    Args:
        file_path: the path to the file to load and deserialize.

    Raises:
        ValueError: if the data is inconsistent with the rest of the dataset or a component is unknown.
        PowerGridError: if there was an internal error.

    Returns:
        The deserialized dataset in Power grid model input format.
    """
    with open(file_path, mode="rb") as file_pointer:
        return msgpack_deserialize(file_pointer.read(), data_filter=data_filter)


def msgpack_serialize_to_file(
    file_path: Path, data: Dataset, dataset_type: DatasetType | None = None, use_compact_list: bool = False
):
    """
    Export msgpack data in most recent format.

    Args:
        file_path: the path to the file to load and deserialize.
        data: a single or batch dataset for power-grid-model.
        use_compact_list: write components on a single line.
        indent: indent of the file, default 2.

    Returns:
        Save to file.
    """
    data = _map_to_component_types(data)
    result = msgpack_serialize(data=data, dataset_type=dataset_type, use_compact_list=use_compact_list)

    with open(file_path, mode="wb") as file_pointer:
        file_pointer.write(result)


def import_json_data(json_file: Path, data_type: str, *args, **kwargs) -> Dataset:
    """
    [deprecated] Import json data.

    **WARNING:** This function is deprecated. Please use json_deserialize_from_file instead.

    Args:
        json_file: path to the json file.
        data_type: type of data: input, update, sym_output, or asym_output.
        [deprecated]: All extra positional and keyword arguments are ignored.

    Returns:
        A single or batch dataset for power-grid-model.
    """
    warnings.warn(_DEPRECATED_JSON_DESERIALIZATION_MSG, DeprecationWarning)
    if args:
        warnings.warn("Provided positional arguments at index 2 and following are deprecated.", DeprecationWarning)
    if kwargs:
        warnings.warn(f"Provided keyword arguments {list(kwargs.keys())} are deprecated.", DeprecationWarning)

    return _compatibility_deprecated_import_json_data(json_file=json_file, data_type=data_type)  # type: ignore


def export_json_data(
    json_file: Path, data: Dataset, indent: int | None = 2, compact: bool = False, use_deprecated_format: bool = True
):
    """
    [deprecated] Export json data in a deprecated serialization format.

    **WARNING:** This function is deprecated. Please use json_serialize_to_file instead.

    For backwards compatibility, this function outputs the deprecated serialization format by default.
    This feature may be removed in the future.

    Args:
        json_file: path to json file.
        data: a single or batch dataset for power-grid-model.
        indent: indent of the file, default 2.
        compact: write components on a single line.
        use_deprecated_format: use the old style format. Defaults to True for backwards compatibility.

    Returns:
        Save to file.
    """
    warnings.warn(_DEPRECATED_JSON_SERIALIZATION_MSG, DeprecationWarning)
    if use_deprecated_format:
        warnings.warn(
            "Argument use_deprecated_format is a temporary backwards-compatibility measure. "
            "Please upgrade to use_deprecated_format=False or json_serialize_to_file as soon as possible.",
            DeprecationWarning,
            stacklevel=2,
        )
        _compatibility_deprecated_export_json_data(json_file=json_file, data=data)
    else:
        json_serialize_to_file(file_path=json_file, data=data, use_compact_list=compact, indent=indent)


def _compatibility_deprecated_export_json_data(
    json_file: Path, data: Dataset, indent: int | None = 2, compact: bool = False
):
    serialized_data = json_serialize(data=data, use_compact_list=compact, indent=-1 if indent is None else indent)
    old_format_serialized_data = json.dumps(json.loads(serialized_data)["data"])
    with open(json_file, mode="w", encoding="utf-8") as file_pointer:
        file_pointer.write(old_format_serialized_data)


def import_input_data(json_file: Path) -> SingleDataset:
    """
    [deprecated] Import input json data.

    **WARNING:** This function is deprecated. Please use json_deserialize_from_file instead.

    For backwards and forward compatibility, this function supportes both the latest and the old serialization format.

    Args:
        json_file: path to the json file.

    Returns:
        A single dataset for power-grid-model.
    """
    warnings.warn(_DEPRECATED_JSON_DESERIALIZATION_MSG, DeprecationWarning)

    data = _compatibility_deprecated_import_json_data(json_file=json_file, data_type=DatasetType.input)
    assert isinstance(data, dict)
    assert all(isinstance(component, np.ndarray) and component.ndim == 1 for component in data.values())
    return cast_type(SingleDataset, data)


def import_update_data(json_file: Path) -> BatchDataset:
    """
    [deprecated] Import update json data.

    **WARNING:** This function is deprecated. Please use json_deserialize_from_file instead.

    For backwards and forward compatibility, this function supportes both the latest and the old serialization format.

    Args:
        json_file: path to the json file.

    Returns:
        A batch dataset for power-grid-model.
    """
    warnings.warn(_DEPRECATED_JSON_DESERIALIZATION_MSG, DeprecationWarning)

    return cast_type(
        BatchDataset,
        _compatibility_deprecated_import_json_data(json_file=json_file, data_type=DatasetType.update),
    )


def _compatibility_deprecated_import_json_data(json_file: Path, data_type: DatasetType):
    with open(json_file, mode="r", encoding="utf-8") as file_pointer:
        data = json.load(file_pointer)

    if "version" not in data:  # convert old format to version 1.0
        data = {
            "attributes": {},
            "data": data,
            "is_batch": isinstance(data, list),
            "type": data_type,
            "version": "1.0",
        }

    result = json_deserialize(json.dumps(data))
    if get_dataset_type(result) != data_type:
        raise PowerGridSerializationError("An internal error occured during deserialization")

    return result


def self_test():
    """
    Perform a self-test of the Power Grid Model functionality.

    Tests whether the installation was successful and there are no build errors,
    segmentation violations, undefined symbols, etc.

    This function is designed to validate the basic functionality of data serialization,
    model instantiation, power flow calculation, and serialization of results using the
    Power Grid Model library.

    Raises:
        PowerGridError: if there was an internal error.
    """
    with tempfile.TemporaryDirectory() as temp_dir:
        # Create a simple JSON input data file in the temporary directory
        input_data = {
            "version": "1.0",
            "type": "input",
            "is_batch": False,
            "attributes": {},
            "data": {
                "node": [{"id": 1, "u_rated": 10000}],
                "source": [{"id": 2, "node": 1, "u_ref": 1, "sk": 1e20}],
                "sym_load": [{"id": 3, "node": 1, "status": 1, "type": 0, "p_specified": 0, "q_specified": 0}],
            },
        }

        input_file_path = Path(temp_dir) / "input_data.json"
        input_file_path.write_text(json.dumps(input_data))

        try:
            # Load the created JSON input data file (deserialize)
            deserialized_data = json_deserialize_from_file(input_file_path)

            # Create a PowerGridModel instance from the loaded input data
            model = PowerGridModel(deserialized_data)

            # Run a simple power flow calculation on the created model (linear calculation)
            output_data = model.calculate_power_flow(calculation_method=CalculationMethod.linear)

            # Write the calculation result to a file in the temporary directory
            output_file_path = Path(temp_dir) / "output_data.json"

            json_serialize_to_file(output_file_path, output_data)

            # Verify that the written output is correct
            with open(output_file_path, "r", encoding="utf-8") as output_file:
                output_data = json.load(output_file)

            assert output_data is not None
            assert math.isclose(
                output_data["data"]["node"][0]["u"], input_data["data"]["node"][0]["u_rated"], abs_tol=1e-9
            )

            print("Self test finished.")
        except Exception as e:
            raise PowerGridError from e



# SPDX-FileCopyrightText: Contributors to the Power Grid Model project <powergridmodel@lfenergy.org>
#
# SPDX-License-Identifier: MPL-2.0

"""
Power grid model buffer handler tests
"""

import numpy as np
import pytest

from power_grid_model._core.buffer_handling import _get_dense_buffer_properties, _get_sparse_buffer_properties
from power_grid_model._core.dataset_definitions import ComponentType, DatasetType
from power_grid_model._core.power_grid_meta import initialize_array, power_grid_meta_data


def load_data(component_type, is_batch, is_sparse, is_columnar):
    """Creates load data of different formats for testing"""
    shape = (2, 4) if is_batch else (4,)
    load = initialize_array(DatasetType.update, component_type, shape)
    columnar_names = ["p_specified", "q_specified"]

    if is_columnar:
        if is_sparse:
            return {"indptr": np.array([0, 5, 8]), "data": {k: load.reshape(-1)[k] for k in columnar_names}}
        return {k: load[k] for k in columnar_names}
    if is_sparse:
        return {"indptr": np.array([0, 5, 8]), "data": load.reshape(-1)}
    return load


@pytest.mark.parametrize(
    "component_type, is_batch, is_columnar",
    [
        pytest.param(ComponentType.sym_load, True, True, id="sym_load-batch-columnar"),
        pytest.param(ComponentType.sym_load, True, False, id="sym_load-batch-row_based"),
        pytest.param(ComponentType.sym_load, False, True, id="sym_load-single-columnar"),
        pytest.param(ComponentType.sym_load, False, False, id="sym_load-single-row_based"),
        pytest.param(ComponentType.asym_load, True, True, id="asym_load-batch-columnar"),
        pytest.param(ComponentType.asym_load, True, False, id="asym_load-batch-row_based"),
        pytest.param(ComponentType.asym_load, False, True, id="asym_load-single-columnar"),
        pytest.param(ComponentType.asym_load, False, False, id="asym_load-single-row_based"),
    ],
)
def test__get_dense_buffer_properties(component_type, is_batch, is_columnar):
    data = load_data(component_type, is_batch=is_batch, is_columnar=is_columnar, is_sparse=False)
    schema = power_grid_meta_data[DatasetType.update][component_type]
    batch_size = 2 if is_batch else None
    properties = _get_dense_buffer_properties(data, schema=schema, is_batch=is_batch, batch_size=batch_size)

    assert not properties.is_sparse
    assert properties.is_batch == is_batch
    assert properties.batch_size == (2 if is_batch else 1)
    assert properties.n_elements_per_scenario == 4
    assert properties.n_total_elements == 8 if is_batch else 4
    if is_columnar:
        assert properties.columns == list(data.keys())
    else:
        assert properties.columns is None


@pytest.mark.parametrize(
    "component_type, is_columnar",
    [
        pytest.param(ComponentType.sym_load, True, id="sym_load-columnar"),
        pytest.param(ComponentType.sym_load, False, id="sym_load-row_based"),
        pytest.param(ComponentType.asym_load, True, id="asym_load-columnar"),
        pytest.param(ComponentType.asym_load, False, id="asym_load-row_based"),
    ],
)
def test__get_sparse_buffer_properties(component_type, is_columnar):
    data = load_data(component_type, is_batch=True, is_columnar=is_columnar, is_sparse=True)

    schema = power_grid_meta_data[DatasetType.update][component_type]
    properties = _get_sparse_buffer_properties(data, schema=schema, batch_size=2)

    assert properties.is_sparse
    assert properties.is_batch
    assert properties.batch_size == 2
    assert properties.n_elements_per_scenario == -1
    assert properties.n_total_elements == 8
    if is_columnar:
        assert properties.columns == list(data["data"].keys())
    else:
        assert properties.columns is None
